
From: Bodo Stroesser <bstroesser@fujitsu-siemens.com>

s390 has fast read access to realtime clock (nanosecond resolution).  So it
makes sense to have an arch-specific implementation not only of __delay, but
__udelay also.

Signed-off-by: Bodo Stroesser <bstroesser@fujitsu-siemens.com>
Signed-off-by: Jeff Dike <jdike@addtoit.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/um/kernel/ksyms.c     |    3 ---
 arch/um/kernel/time_kern.c |   18 ------------------
 arch/um/sys-i386/delay.c   |   18 ++++++++++++++++++
 arch/um/sys-i386/ksyms.c   |    5 +++++
 arch/um/sys-x86_64/delay.c |   18 ++++++++++++++++++
 include/asm-um/delay.h     |    2 ++
 6 files changed, 43 insertions(+), 21 deletions(-)

diff -puN arch/um/kernel/ksyms.c~uml-s390-preparation-delay-moved-to-arch arch/um/kernel/ksyms.c
--- 25/arch/um/kernel/ksyms.c~uml-s390-preparation-delay-moved-to-arch	Mon May  2 17:15:49 2005
+++ 25-akpm/arch/um/kernel/ksyms.c	Mon May  2 17:15:49 2005
@@ -10,7 +10,6 @@
 #include "linux/spinlock.h"
 #include "linux/highmem.h"
 #include "asm/current.h"
-#include "asm/delay.h"
 #include "asm/processor.h"
 #include "asm/unistd.h"
 #include "asm/pgalloc.h"
@@ -28,8 +27,6 @@ EXPORT_SYMBOL(uml_physmem);
 EXPORT_SYMBOL(set_signals);
 EXPORT_SYMBOL(get_signals);
 EXPORT_SYMBOL(kernel_thread);
-EXPORT_SYMBOL(__const_udelay);
-EXPORT_SYMBOL(__udelay);
 EXPORT_SYMBOL(sys_waitpid);
 EXPORT_SYMBOL(task_size);
 EXPORT_SYMBOL(flush_tlb_range);
diff -puN arch/um/kernel/time_kern.c~uml-s390-preparation-delay-moved-to-arch arch/um/kernel/time_kern.c
--- 25/arch/um/kernel/time_kern.c~uml-s390-preparation-delay-moved-to-arch	Mon May  2 17:15:49 2005
+++ 25-akpm/arch/um/kernel/time_kern.c	Mon May  2 17:15:49 2005
@@ -48,8 +48,6 @@ static unsigned long long prev_usecs;
 static long long delta;   		/* Deviation per interval */
 #endif
 
-#define MILLION 1000000
-
 void timer_irq(union uml_pt_regs *regs)
 {
 	unsigned long long ticks = 0;
@@ -136,22 +134,6 @@ long um_stime(int __user *tptr)
 	return 0;
 }
 
-void __udelay(unsigned long usecs)
-{
-	int i, n;
-
-	n = (loops_per_jiffy * HZ * usecs) / MILLION;
-	for(i=0;i<n;i++) ;
-}
-
-void __const_udelay(unsigned long usecs)
-{
-	int i, n;
-
-	n = (loops_per_jiffy * HZ * usecs) / MILLION;
-	for(i=0;i<n;i++) ;
-}
-
 void timer_handler(int sig, union uml_pt_regs *regs)
 {
 	local_irq_disable();
diff -puN arch/um/sys-i386/delay.c~uml-s390-preparation-delay-moved-to-arch arch/um/sys-i386/delay.c
--- 25/arch/um/sys-i386/delay.c~uml-s390-preparation-delay-moved-to-arch	Mon May  2 17:15:49 2005
+++ 25-akpm/arch/um/sys-i386/delay.c	Mon May  2 17:15:49 2005
@@ -1,3 +1,6 @@
+#include "linux/delay.h"
+#include "asm/param.h"
+
 void __delay(unsigned long time)
 {
 	/* Stolen from the i386 __loop_delay */
@@ -12,3 +15,18 @@ void __delay(unsigned long time)
 		:"0" (time));
 }
 
+void __udelay(unsigned long usecs)
+{
+	int i, n;
+
+	n = (loops_per_jiffy * HZ * usecs) / MILLION;
+	for(i=0;i<n;i++) ;
+}
+
+void __const_udelay(unsigned long usecs)
+{
+	int i, n;
+
+	n = (loops_per_jiffy * HZ * usecs) / MILLION;
+	for(i=0;i<n;i++) ;
+}
diff -puN arch/um/sys-i386/ksyms.c~uml-s390-preparation-delay-moved-to-arch arch/um/sys-i386/ksyms.c
--- 25/arch/um/sys-i386/ksyms.c~uml-s390-preparation-delay-moved-to-arch	Mon May  2 17:15:49 2005
+++ 25-akpm/arch/um/sys-i386/ksyms.c	Mon May  2 17:15:49 2005
@@ -2,6 +2,7 @@
 #include "linux/in6.h"
 #include "linux/rwsem.h"
 #include "asm/byteorder.h"
+#include "asm/delay.h"
 #include "asm/semaphore.h"
 #include "asm/uaccess.h"
 #include "asm/checksum.h"
@@ -14,3 +15,7 @@ EXPORT_SYMBOL(__up_wakeup);
 
 /* Networking helper routines. */
 EXPORT_SYMBOL(csum_partial);
+
+/* delay core functions */
+EXPORT_SYMBOL(__const_udelay);
+EXPORT_SYMBOL(__udelay);
diff -puN arch/um/sys-x86_64/delay.c~uml-s390-preparation-delay-moved-to-arch arch/um/sys-x86_64/delay.c
--- 25/arch/um/sys-x86_64/delay.c~uml-s390-preparation-delay-moved-to-arch	Mon May  2 17:15:49 2005
+++ 25-akpm/arch/um/sys-x86_64/delay.c	Mon May  2 17:15:49 2005
@@ -5,7 +5,9 @@
  * Licensed under the GPL
  */
 
+#include "linux/delay.h"
 #include "asm/processor.h"
+#include "asm/param.h"
 
 void __delay(unsigned long loops)
 {
@@ -14,6 +16,22 @@ void __delay(unsigned long loops)
 	for(i = 0; i < loops; i++) ;
 }
 
+void __udelay(unsigned long usecs)
+{
+	int i, n;
+
+	n = (loops_per_jiffy * HZ * usecs) / MILLION;
+	for(i=0;i<n;i++) ;
+}
+
+void __const_udelay(unsigned long usecs)
+{
+	int i, n;
+
+	n = (loops_per_jiffy * HZ * usecs) / MILLION;
+	for(i=0;i<n;i++) ;
+}
+
 /*
  * Overrides for Emacs so that we follow Linus's tabbing style.
  * Emacs will notice this stuff at the end of the file and automatically
diff -puN include/asm-um/delay.h~uml-s390-preparation-delay-moved-to-arch include/asm-um/delay.h
--- 25/include/asm-um/delay.h~uml-s390-preparation-delay-moved-to-arch	Mon May  2 17:15:49 2005
+++ 25-akpm/include/asm-um/delay.h	Mon May  2 17:15:49 2005
@@ -4,4 +4,6 @@
 #include "asm/arch/delay.h"
 #include "asm/archparam.h"
 
+#define MILLION 1000000
+
 #endif
_
